# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""create healthmanager table

Revision ID: 92fe9857279
Revises: 256852d5ff7c
Create Date: 2015-01-22 16:58:23.440247

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '92fe9857279'
down_revision = '256852d5ff7c'


def upgrade():
    op.create_table(
        u'amphora_health',
        sa.Column(u'amphora_id', sa.String(36), nullable=False,
                  primary_key=True),
        sa.Column(u'last_update', sa.DateTime(timezone=True),
                  nullable=False)

    )
